package net.sergeych.crypto2

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import kotlinx.serialization.Transient

@Serializable
sealed class UniversalKey : DecryptingKey {

    abstract val tag: KeyTag



    @Serializable
    @SerialName("sy")
    data class Symmetric(val key: SymmetricKey) : UniversalKey(), EncryptingKey by key, DecryptingKey by key {
        @Transient
        override val tag: KeyTag = key.encryptingTag

        @Transient
        override val nonceBytesLength: Int = key.nonceBytesLength

        override fun toString() = "U.Sym:$tag"
    }

    @Serializable
    @SerialName("sn")
    data class Session(val key: SafeKeyExchange.SessionKey) : UniversalKey(), EncryptingKey by key,
        DecryptingKey by key {
        @Transient
        override val tag: KeyTag = key.encryptingTag
        @Transient
        override val nonceBytesLength: Int = key.nonceBytesLength
    }

    @Serializable
    @SerialName("se")
    data class Secret(val key: Asymmetric.SecretKey) : UniversalKey(), DecryptingKey by key {
        override val tag: KeyTag by lazy { key.decryptingTag }
        override fun toString() = "U.Sec:$tag"
    }



    companion object {
        fun from(key: DecryptingKey) =
            when (key) {
                is Asymmetric.SecretKey -> Secret(key)
                is SymmetricKey -> Symmetric(key)
                is SafeKeyExchange.SessionKey -> Session(key)
                else -> throw UnsupportedOperationException("can't create universal key from ${key::class.simpleName}")
            }
    }

}
